package cn.estsh.i3plus.ext.mes.pcn.pojo.model.jx;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiParam;
import lombok.Data;

import java.io.Serializable;

/**
 * @author wangjie
 * @version 1.0
 * @date 2021/1/15 15:32
 **/
@Data
@ApiModel("嘉兴松下工位扫描 MODEL")
public class JxSnWorkCellScanModel implements Serializable {

    private static final long serialVersionUID = 8683799945479864228L;

    @ApiParam("序号")
    private Integer index;

    @ApiParam("操作时间")
    private String modifyDatetime;

    @ApiParam("操作人")
    private String modifyUser;

    @ApiParam("管理码")
    private String manageCode;

    @ApiParam("过程条码")
    private String serialNumber;

    @ApiParam("产品条码")
    private String productSn;

    @ApiParam("状态")
    private String statusName;

    @ApiParam("质量状态")
    private String qcStatusName;

    @ApiParam("颜色")
    private String color;

    public JxSnWorkCellScanModel index(Integer index) {
        this.index = index;
        return this;
    }

    public JxSnWorkCellScanModel manageCode(String manageCode) {
        this.manageCode = manageCode;
        return this;
    }

    public JxSnWorkCellScanModel statusName(String statusName) {
        this.statusName = statusName;
        return this;
    }

    public JxSnWorkCellScanModel qcStatusName(String qcStatusName) {
        this.qcStatusName = qcStatusName;
        return this;
    }

    public JxSnWorkCellScanModel color(String color) {
        this.color = color;
        return this;
    }

}
